'use strict';

module.exports = function (Tuio, _) {
  Tuio.Object = Tuio.Container.extend({
    symbolId: null,
    angle: null,
    rotationSpeed: null,
    rotationAccel: null,

    initialize: function (params) {
      Tuio.Container.prototype.initialize.call(this, params);

      this.symbolId = params.sym;
      this.angle = params.a;
      this.rotationSpeed = 0;
      this.rotationAccel = 0;
    },

    update: function (params) {
      var lastPoint = this.path[this.path.length - 1];
      Tuio.Container.prototype.update.call(this, params);

      if (
        params.hasOwnProperty("rs") &&
        params.hasOwnProperty("ra")) {

        this.angle = params.a;
        this.rotationSpeed = params.rs;
        this.rotationAccel = params.ra;
      } else {
        var diffTime = this.currentTime.subtractTime(lastPoint.getTuioTime()),
        dt = diffTime.getTotalMilliseconds() / 1000,
        lastAngle = this.angle,
        lastRotationSpeed = this.rotationSpeed;
        this.angle = params.a;

        var da = (this.angle - lastAngle) / (2 * Math.PI);
        if (da > 0.75) {
          da -= 1;
        } else if (da < -0.75) {
          da += 1;
        }
        
        this.rotationSpeed = da / dt;
        this.rotationAccel = (this.rotationSpeed - lastRotationSpeed) / dt;
      }

      this.updateObjectState();
    },

    updateObject: function (tobj) {
      Tuio.Container.prototype.updateContainer.call(this, tobj);

      this.angle = tobj.getAngle();
      this.rotationSpeed = tobj.getRotationSpeed();
      this.rotationAccel = tobj.getRotationAccel();
      
      this.updateObjectState();
    },

    updateObjectState: function () {
      if ((this.rotationAccel !== 0) && (this.state !== Tuio.Object.TUIO_STOPPED)) {
        this.state = Tuio.Object.TUIO_ROTATING;
      }
    },

    stop: function (ttime) {
      this.update({
        ttime: ttime,
        xp: this.xPos,
        yp: this.yPos,
        a: this.angle
      });
    },

    getSymbolId: function () {
      return this.symbolId;
    },

    getAngle: function () {
      return this.angle;
    },

    getAngleDegrees: function () {
      return this.angle / Math.PI * 180;
    },

    getRotationSpeed: function () {
      return this.rotationSpeed;
    },

    getRotationAccel: function () {
      return this.rotationAccel;
    },

    isMoving: function () {
      return (
        (this.state === Tuio.Object.TUIO_ACCELERATING) ||
        (this.state === Tuio.Object.TUIO_DECELERATING) ||
        (this.state === Tuio.Object.TUIO_ROTATING)
      );
    }
  }, {
    TUIO_ROTATING: 5,

    fromObject: function (tobj) {
      return new Tuio.Object({
        xp: tobj.getX(),
        yp: tobj.getY(),
        si: tobj.getSessionID(),
        sym: tobj.getSymbolId(),
        a: tobj.getAngle()
      });
    }
  });
}