'use strict';

module.exports = function (Tuio, _) {
  Tuio.Time = Tuio.Model.extend({
    seconds: 0,
    microSeconds: 0,

    initialize: function (sec, usec) {
      this.seconds = sec || 0;
      this.microSeconds = usec || 0;
    },

    add: function (us) {
      return new Tuio.Time(
        this.seconds + Math.floor(us / 1000000),
        this.microSeconds + us % 1000000
      );
    },

    addTime: function (ttime) {
      var sec = this.seconds + ttime.getSeconds(),
      usec = this.microSeconds + ttime.getMicroseconds();
      sec += Math.floor(usec / 1000000);
      usec = usec % 1000000;

      return new Tuio.Time(sec, usec);
    },

    subtract: function (us) {
      var sec = this.seconds - Math.floor(us / 1000000),
      usec = this.microSeconds - us % 1000000;

      if (usec < 0) {
        usec += 1000000;
        sec = sec - 1;
      }

      return new Tuio.Time(sec, usec);
    },

    subtractTime: function (ttime) {
      var sec = this.seconds - ttime.getSeconds(),
      usec = this.microSeconds - ttime.getMicroseconds();

      if (usec < 0) {
        usec += 1000000;
        sec = sec - 1;
      }

      return new Tuio.Time(sec, usec);
    },

    equals: function (ttime) {
      return (
        (this.seconds === ttime.getSeconds()) &&
        (this.microSeconds === ttime.getMicroseconds())
      );
    },

    reset: function () {
      this.seconds = 0;
      this.microSeconds = 0;
    },

    getSeconds: function () {
      return this.seconds;
    },

    getMicroseconds: function () {
      return this.microSeconds;
    },

    getTotalMilliseconds: function () {
      return this.seconds * 1000 + Math.floor(this.microSeconds / 1000);
    }
  }, {
    startSeconds: 0,
    startMicroSeconds: 0,

    fromMilliseconds: function (msec) {
      return new Tuio.Time(
        Math.floor(msec / 1000),
        1000 * (msec % 1000)
      );
    },

    fromTime: function (ttime) {
      return new Tuio.Time(
        ttime.getSeconds(),
        ttime.getMicroseconds()
      );
    },

    initSession: function () {
      var startTime = Tuio.Time.getSystemTime();
      Tuio.Time.startSeconds = startTime.getSeconds();
      Tuio.Time.startMicroSeconds = startTime.getMicroseconds();
    },

    getSessionTime: function () {
      return Tuio.Time.getSystemTime().subtractTime(Tuio.Time.getStartTime());
    },

    getStartTime: function () {
      return new Tuio.Time(
        Tuio.Time.startSeconds,
        Tuio.Time.startMicroSeconds
      );
    },

    getSystemTime: function () {
      var usec = new Date().getTime() * 1000;

      return new Tuio.Time(
        Math.floor(usec / 1000000),
        usec % 1000000
      );
    }
  });
};
